@@ -213,7 +213,7 @@ bool aiounicast_select::Send
 	mpz_t tmp;
 	mpz_init_set(tmp, m);
 	if (aio_is_encrypted)
-		mpz_add(tmp, tmp, aio_hide_length);
+		mpz_add(tmp, tmp, aio_hide_length); // add $2^c$ to hide length
 	size_t size = mpz_sizeinbase(tmp, TMCG_MPZ_IO_BASE);
 	if ((size * 2) >= buf_in_size)
 	{
@@ -235,7 +235,7 @@ bool aiounicast_select::Send
 	}
 	else
 	{
-		std::cerr << "aiounicast_select: realsize does not fit" << std::endl;
+		std::cerr << "aiounicast_select(" << j << "): realsize does not fit" << std::endl;
 		delete [] buf;
 		return false;
 	}
@@ -250,7 +250,7 @@ bool aiounicast_select::Send
 	if (aio_is_encrypted)
 	{
 		memmove(buf + 1, buf, realsize - 1);
-		buf[0] = '+'; // set plus-character als non-zero prefix
+		buf[0] = '+'; // use plus-character as a non-zero prefix
 		err = gcry_cipher_encrypt(*enc_out[i_in], buf + 1, realsize - 1, NULL, 0);
 		if (err)
 		{
@@ -279,7 +279,7 @@ bool aiounicast_select::Send
 		}
 		else
 		{
-			std::cerr << "aiounicast_select: realsize does not fit" << std::endl;
+			std::cerr << "aiounicast_select(" << j << "): realsize does not fit" << std::endl;
 			delete [] buf;
 			return false;
 		}
@@ -344,7 +344,7 @@ bool aiounicast_select::Send
 				iv_flag_out[i_in] = true; // IV has been sent
 		}
 	}
-	// calculate the MAC over all data including line delimiter
+	// calculate the MAC over all data including the line delimiter
 	if (aio_is_authenticated)
 	{
 		err = gcry_mac_write(*mac_out[i_in], buf, realsize);
@@ -401,7 +401,7 @@ bool aiounicast_select::Send
 			realnum += num;
 		}
 		else
-			std::cerr << "WARNING: aiounicast_select FD_ISSET not true" << std::endl;
+			std::cerr << "WARNING: aiounicast_select(" << j << ") FD_ISSET not true" << std::endl;
 	}
 	while ((realnum < realsize) && (time(NULL) < (entry_time + timeout)));
 	delete [] buf;
