@@ -185,7 +185,7 @@ int main
 	for (size_t i = 0; i < n; i++)
 	{
 		mpz_clear(mp[i]);
-		delete mp[i];
+		delete [] mp[i];
 	}
 	mp.clear();
 	delete com, delete com2;
@@ -230,7 +230,7 @@ int main
 	assert(com2->Verify(a, b, mp));
 	{
 		mpz_clear(mp[0]);
-		delete mp[0];
+		delete [] mp[0];
 	}
 	mp.clear();
 	delete com, delete com2;
@@ -425,11 +425,11 @@ int main
 			for (size_t i = 0; i < n; i++)
 			{
 				mpz_clear(m[i]), mpz_clear(m_pi[i]), mpz_clear(R[i]);
-				delete m[i], delete m_pi[i], delete R[i];
+				delete [] m[i], delete [] m_pi[i], delete [] R[i];
 				mpz_clear(e[i].first), mpz_clear(e[i].second);
-				delete e[i].first, delete e[i].second;
+				delete [] e[i].first, delete [] e[i].second;
 				mpz_clear(E[i].first), mpz_clear(E[i].second);
-				delete E[i].first, delete E[i].second;
+				delete [] E[i].first, delete [] E[i].second;
 			}
 			m.clear(), m_pi.clear(), R.clear(), e.clear(), E.clear();
 			mpz_clear(c), mpz_clear(r);
@@ -574,9 +574,9 @@ int main
 				mpz_clear(m[i]);
 				mpz_clear(e[i].first), mpz_clear(e[i].second);
 				mpz_clear(E[i].first), mpz_clear(E[i].second);
-				delete m[i];
-				delete e[i].first, delete e[i].second;
-				delete E[i].first, delete E[i].second;
+				delete [] m[i];
+				delete [] e[i].first, delete [] e[i].second;
+				delete [] E[i].first, delete [] E[i].second;
 			}
 			m.clear(), e.clear(), E.clear();
 			delete vsshe, delete edcf, delete skc, delete com;
